@@ -119,31 +119,41 @@ COMPRESSORS = {
 ---
 -- RFC 4492 section 5.1.1 "Supported Elliptic Curves Extension".
 ELLIPTIC_CURVES = {
-  sect163k1 = 1,
-  sect163r1 = 2,
-  sect163r2 = 3,
-  sect193r1 = 4,
-  sect193r2 = 5,
-  sect233k1 = 6,
-  sect233r1 = 7,
-  sect239k1 = 8,
-  sect283k1 = 9,
-  sect283r1 = 10,
-  sect409k1 = 11,
-  sect409r1 = 12,
-  sect571k1 = 13,
-  sect571r1 = 14,
-  secp160k1 = 15,
-  secp160r1 = 16,
-  secp160r2 = 17,
-  secp192k1 = 18,
-  secp192r1 = 19,
-  secp224k1 = 20,
-  secp224r1 = 21,
-  secp256k1 = 22,
+  sect163k1 = 1, --deprecated
+  sect163r1 = 2, --deprecated
+  sect163r2 = 3, --deprecated
+  sect193r1 = 4, --deprecated
+  sect193r2 = 5, --deprecated
+  sect233k1 = 6, --deprecated
+  sect233r1 = 7, --deprecated
+  sect239k1 = 8, --deprecated
+  sect283k1 = 9, --deprecated
+  sect283r1 = 10, --deprecated
+  sect409k1 = 11, --deprecated
+  sect409r1 = 12, --deprecated
+  sect571k1 = 13, --deprecated
+  sect571r1 = 14, --deprecated
+  secp160k1 = 15, --deprecated
+  secp160r1 = 16, --deprecated
+  secp160r2 = 17, --deprecated
+  secp192k1 = 18, --deprecated
+  secp192r1 = 19, --deprecated
+  secp224k1 = 20, --deprecated
+  secp224r1 = 21, --deprecated
+  secp256k1 = 22, --deprecated
   secp256r1 = 23,
   secp384r1 = 24,
   secp521r1 = 25,
+  brainpoolP256r1 = 26, --RFC7027
+  brainpoolP384r1 = 27,
+  brainpoolP512r1 = 28,
+  ecdh_x25519 = 29, -- draft rfc4492
+  ecdh_x448 = 30, --draft rfc4492
+  ffdhe2048 = 256, --RFC7919
+  ffdhe3072 = 257, --RFC7919
+  ffdhe4096 = 258, --RFC7919
+  ffdhe6144 = 259, --RFC7919
+  ffdhe8192 = 260, --RFC7919
   arbitrary_explicit_prime_curves = 0xFF01,
   arbitrary_explicit_char2_curves = 0xFF02,
 }
@@ -166,12 +176,15 @@ HashAlgorithms = {
   sha256 = 4,
   sha384 = 5,
   sha512 = 6,
+  intrinsic = 8,
 }
 SignatureAlgorithms = {
   anonymous = 0,
   rsa = 1,
   dsa = 2,
   ecdsa = 3,
+  ed25519 = 7,
+  ed448 = 8,
 }
 
 ---
@@ -202,6 +215,10 @@ EXTENSIONS = {
   ["client_certificate_type"] = 19,
   ["server_certificate_type"] = 20,
   ["padding"] = 21, -- Temporary, expires 2015-03-12
+  ["encrypt_then_mac"] = 22, -- rfc7366
+  ["extended_master_secret"] = 23, -- rfc7627
+  ["token_binding"] = 24, -- Temporary, expires 2018-02-04
+  ["cached_info"] = 25, -- rfc7924
   ["SessionTicket TLS"] = 35,
   ["next_protocol_negotiation"] = 13172,
   ["renegotiation_info"] = 65281,
@@ -609,9 +626,20 @@ CIPHERS = {
 ["TLS_PSK_WITH_AES_256_CCM_8"]                     =  0xC0A9,
 ["TLS_PSK_DHE_WITH_AES_128_CCM_8"]                 =  0xC0AA,
 ["TLS_PSK_DHE_WITH_AES_256_CCM_8"]                 =  0xC0AB,
-["TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256"]    =  0xCC13,
-["TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256"]  =  0xCC14,
-["TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256"]      =  0xCC15,
+["TLS_ECDHE_ECDSA_WITH_AES_128_CCM"]               =  0xC0AC,
+["TLS_ECDHE_ECDSA_WITH_AES_256_CCM"]               =  0xC0AD,
+["TLS_ECDHE_ECDSA_WITH_AES_128_CCM_8"]             =  0xC0AE,
+["TLS_ECDHE_ECDSA_WITH_AES_256_CCM_8"]             =  0xC0AF,
+["TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256-draft"]    =  0xCC13, -- RFC7905 superseded
+["TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256-draft"]  =  0xCC14, -- RFC7905 superseded
+["TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256-draft"]      =  0xCC15, -- RFC7905 superseded
+["TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256"]    =  0xCCA8,
+["TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256"]  =  0xCCA9,
+["TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256"]      =  0xCCAA,
+["TLS_PSK_WITH_CHACHA20_POLY1305_SHA256"]          =  0xCCAB,
+["TLS_ECDHE_PSK_WITH_CHACHA20_POLY1305_SHA256"]    =  0xCCAC,
+["TLS_DHE_PSK_WITH_CHACHA20_POLY1305_SHA256"]      =  0xCCAD,
+["TLS_RSA_PSK_WITH_CHACHA20_POLY1305_SHA256"]      =  0xCCAE,
 ["SSL_RSA_FIPS_WITH_DES_CBC_SHA"]                  =  0xFEFE,
 ["SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA"]             =  0xFEFF,
 }
@@ -1159,6 +1187,20 @@ handshake_parse = {
 
         return b, j
       end,
+
+      NewSessionTicket = function (buffer, j, msg_end, protocol)
+        -- Need 4 bytes for parsing.
+        local have = #buffer - j + 1
+        if have < 4 then
+          return nil, j, 4
+        end
+
+        local b = {}
+        -- Parse body.
+        b.ticket_lifetime_hint, b.ticket, j = unpack(">I4 s2", buffer, j)
+
+        return b, j
+      end,
 }
 
 message_parse = {
@@ -1253,11 +1295,13 @@ end
 ---
 -- Read a SSL/TLS record
 -- @param buffer   The read buffer
--- @param i        The position in the buffer to start reading
+-- @param i        The position in the buffer to start reading (default: 1)
 -- @param fragment Message fragment left over from previous record (nil if none)
 -- @return The current position in the buffer
 -- @return The record that was read, as a table
 function record_read(buffer, i, fragment)
+  i = i or 1
+
   -- Ensure we have enough data for the header.
   if #buffer - i < TLS_RECORD_HEADER_LENGTH then
     return i, nil
@@ -1347,7 +1391,8 @@ end
 -- Build a client_hello message
 --
 -- The options table has the following keys:
--- * <code>"protocol"</code> - The TLS protocol version string
+-- * <code>"protocol"</code> - The TLS protocol version string for the client_hello. This indicates the highest protocol version supported.
+-- * <code>"record_protocol"</code> - The TLS protocol version string for the TLS record. This indicates the lowest protocol version supported.
 -- * <code>"ciphers"</code> - a table containing the cipher suite names. Defaults to the NULL cipher
 -- * <code>"compressors"</code> - a table containing the compressor names. Default: NULL
 -- * <code>"extensions"</code> - a table containing the extension names. Default: no extensions
@@ -1374,7 +1419,8 @@ function client_hello(t)
   table.insert(b, stdnse.generate_random_string(28))
 
   -- Set the session ID.
-  table.insert(b, '\0')
+  local sid = t["session_id"] or ""
+  table.insert(b, pack(">s1", sid))
 
   -- Cipher suites.
   ciphers = {}
@@ -1449,7 +1495,7 @@ function client_hello(t)
   table.insert(h, pack(">s3", b))
 
   -- Record layer version should be SSLv3 (lowest compatible record version)
-  return record_write("handshake", "SSLv3", table.concat(h))
+  return record_write("handshake", t.record_protocol or "SSLv3", table.concat(h))
 end
 
 local function read_atleast(s, n)
@@ -1502,4 +1548,20 @@ function record_buffer(sock, buffer, i)
   return true, buffer
 end
 
+-- Get a server_name for use with the TLS Server Name Indication extension.
+--
+-- This returns the value of the script argument "tls.servername" if given.  Otherwise, it
+-- returns the target name of the host parameter.
+--
+-- @param host Host table as received by the action function
+-- @return String of the selected host name
+function servername(host)
+    local script_arg = stdnse.get_script_args("tls.servername")
+    if script_arg then
+        return script_arg
+    elseif type(host) == "table" then
+        return host.targetname
+    end
+end
+
 return _ENV;
